/*************************************************
		ClcdPlus.cpp
		Ver. 1.10

	Copyright (C) 2010-2015,
		MicroFan, Div. PeopleMedia, Inc.
	All rights reserved.
*************************************************/

#include "ClcdPlus.h"

#include <stdio.h>
#include <string.h>
#include "Arduino.h"

/*************************************************
* SpiGpio
*************************************************/

SpiGpio::SpiGpio(uint8_t ss, uint8_t addr)
{
    _cs = ss ;
    digitalWrite(_cs, HIGH) ;
    pinMode(_cs, OUTPUT) ; // ~SS

    pinMode(13, OUTPUT) ; // SCK
    pinMode(12, INPUT) ; // MISO
    pinMode(11, OUTPUT) ; // MOSI

    devAddr = addr ;

#if defined(__AVR_ATmega32U4__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
    digitalWrite(13, HIGH) ;
    digitalWrite(11, HIGH) ;
#else
    pinMode(10, OUTPUT) ; // SS
/*
    if (!(SPCR & (1 << SPE))) { // ɐݒ肳ĂȂ
        SPCR = (1 << SPE) | (1 << MSTR) ;
        SPSR |= (1 << SPI2X) ;
    }
*/
#endif
}

uint8_t
SpiGpio::access_addr8(uint8_t cmd, uint8_t reg, uint8_t data)
{
#if defined(__AVR_ATmega32U4__)
    uint8_t d ;
    int i ;

    digitalWrite(_cs, LOW) ;

    d = cmd | (devAddr << 1) ;
    for (i = 0; i < 8; i++) {
        PORTB = (PORTB & 0x7F) | (d & 0x80) ; // SCK & MO
        PORTC &= 0x7F ; // SCK
        d <<= 1 ;
        PORTC |= 0x80 ; // SCK
    }

    d = reg ;
    for (i = 0; i < 8; i++) {
        PORTB = (PORTB & 0x7F) | (d & 0x80) ; // SCK & MO
        PORTC &= 0x7F ; // SCK
        d <<= 1 ;
        PORTC |= 0x80 ;
    }

    d = data ;
    for (i = 0; i < 8; i++) {
        PORTB = (PORTB & 0x7F) | (d & 0x80) ; // SCK & MO
        PORTC &= 0x7F ; // SCK
        d <<= 1 ;
        PORTC |= 0x80 ;
        d |= (PIND & 0x40) ? 1 : 0 ;
    }

    digitalWrite(_cs, HIGH) ;

    return d ;
#elif defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
    uint8_t d, so ;
    int i ;

    digitalWrite(_cs, LOW) ;

    d = cmd | (devAddr << 1) ;
    for (i = 0; i < 8; i++) {
        so = (d & 0x80) ? 0x20:0 ;
        PORTB = (PORTB & 0x5F) | so ; // SCK & MO
        d <<= 1 ;
        PORTB |= 0x80 ; // SCK
    }

    d = reg ;
    for (i = 0; i < 8; i++) {
        so = (d & 0x80) ? 0x20:0 ;
        PORTB = (PORTB & 0x5F) | so ; // SCK & MO
        d <<= 1 ;
        PORTB |= 0x80 ;
    }

    d = data ;
    for (i = 0; i < 8; i++) {
        so = (d & 0x80) ? 0x20:0 ;
        PORTB = (PORTB & 0x5F) | so ; // SCK & MO
        d <<= 1 ;
        PORTB |= 0x80 ;
        d |= (PINB & 0x40) ? 1 : 0 ;
    }

    digitalWrite(_cs, HIGH) ;

    return d ;
#else
    uint8_t xspcr, xspsr ;

    xspcr = SPCR ;
    xspsr = SPSR ;  
    SPCR = (1 << SPE) | (1 << MSTR) ;
    SPSR |= (1 << SPI2X) ;

    digitalWrite(_cs, LOW) ;

    SPDR = cmd | (devAddr << 1) ;
    while (!(SPSR & (1 << SPIF)))
        ;
    SPDR = reg ;
    while (!(SPSR & (1 << SPIF)))
        ;
    SPDR = data ;
    while (!(SPSR & (1 << SPIF)))
        ;

    digitalWrite(_cs, HIGH) ;

    SPCR = xspcr ;
    SPSR = xspsr ;  

    return SPDR ;
#endif
}

void
SpiGpio::write(uint8_t reg, uint8_t data)
{
    access_addr8(GPIO_WRITE, reg, data) ;
}

uint8_t
SpiGpio::read(uint8_t reg)
{
    return access_addr8(GPIO_READ, reg, 0x00) ;
}


/*************************************************
* ClcdPlus
	݊̎̂LiquidCrystalCp
*************************************************/

// When the display powers up, it is configured as follows:
//
// 1. Display clear
// 2. Function set: 
//    DL = 1; 8-bit interface data 
//    N = 0; 1-line display 
//    F = 0; 5x8 dot character font 
// 3. Display on/off control: 
//    D = 0; Display off 
//    C = 0; Cursor off 
//    B = 0; Blinking off 
// 4. Entry mode set: 
//    I/D = 1; Increment by 1 
//    S = 0; No shift 
//
// Note, however, that resetting the Arduino doesn't reset the LCD, so we
// can't assume that its in that state when a sketch starts (and the
// ClcdPlus constructor is called).

ClcdPlus::ClcdPlus() : gpio(SPI_GPIO_CS, SPI_GPIO_ADDR)
{
  _state = 0x04 ;
  digitalWrite(8, LOW);
  pinMode(8, OUTPUT) ;
  
  _displayfunction = LCD_8BITMODE | LCD_1LINE | LCD_5x8DOTS;
  
  begin(16, 1);  
}

void ClcdPlus::begin(uint8_t cols, uint8_t lines, uint8_t dotsize) {
  if (lines > 1) {
    _displayfunction |= LCD_2LINE;
  }
  _numlines = lines;
  _currline = 0;

  // for some 1 line displays you can select a 10 pixel high font
  if ((dotsize != 0) && (lines == 1)) {
    _displayfunction |= LCD_5x10DOTS;
  }

  // SEE PAGE 45/46 FOR INITIALIZATION SPECIFICATION!
  // according to datasheet, we need at least 40ms after power rises above 2.7V
  // before sending commands. Arduino can turn on way befer 4.5V so we'll wait 50
  delayMicroseconds(50000); 
  
    gpio.write(GPIO_IODIRA, 0x00) ;	// f[^
    gpio.write(GPIO_IODIRB, 0xF0) ;	// AXCb`
    gpio.write(GPIO_GPPUB, 0xF0) ;
    gpio.write(GPIO_GPIOB, _state) ;

  //put the LCD into 8 bit mode
  
    // this is according to the hitachi HD44780 datasheet
    // page 45 figure 23

    // Send function set command sequence
    command(LCD_FUNCTIONSET | _displayfunction);
    delayMicroseconds(4500);  // wait more than 4.1ms

    // second try
    command(LCD_FUNCTIONSET | _displayfunction);
    delayMicroseconds(150);

    // third go
    command(LCD_FUNCTIONSET | _displayfunction);

  // finally, set # lines, font size, etc.
  command(LCD_FUNCTIONSET | _displayfunction);  

  // turn the display on with no cursor or blinking default
  _displaycontrol = LCD_DISPLAYON | LCD_CURSOROFF | LCD_BLINKOFF;  
  display();

  // clear it off
  clear();

  // Initialize to default text direction (for romance languages)
  _displaymode = LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT;
  // set the entry mode
  command(LCD_ENTRYMODESET | _displaymode);
}

/********** high level commands, for the user! */
void ClcdPlus::clear()
{
  command(LCD_CLEARDISPLAY);  // clear display, set cursor position to zero
  delayMicroseconds(2000);  // this command takes a long time!
}

void ClcdPlus::home()
{
  command(LCD_RETURNHOME);  // set cursor position to zero
  delayMicroseconds(2000);  // this command takes a long time!
}

void ClcdPlus::setCursor(uint8_t col, uint8_t row)
{
  int row_offsets[] = { 0x00, 0x40, 0x14, 0x54 };
  if ( row > _numlines ) {
    row = _numlines-1;    // we count rows starting w/0
  }
  
  command(LCD_SETDDRAMADDR | (col + row_offsets[row]));
}

// Turn the display on/off (quickly)
void ClcdPlus::noDisplay() {
  _displaycontrol &= ~LCD_DISPLAYON;
  command(LCD_DISPLAYCONTROL | _displaycontrol);
}
void ClcdPlus::display() {
  _displaycontrol |= LCD_DISPLAYON;
  command(LCD_DISPLAYCONTROL | _displaycontrol);
}

// Turns the underline cursor on/off
void ClcdPlus::noCursor() {
  _displaycontrol &= ~LCD_CURSORON;
  command(LCD_DISPLAYCONTROL | _displaycontrol);
}
void ClcdPlus::cursor() {
  _displaycontrol |= LCD_CURSORON;
  command(LCD_DISPLAYCONTROL | _displaycontrol);
}

// Turn on and off the blinking cursor
void ClcdPlus::noBlink() {
  _displaycontrol &= ~LCD_BLINKON;
  command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void ClcdPlus::blink() {
  _displaycontrol |= LCD_BLINKON;
  command(LCD_DISPLAYCONTROL | _displaycontrol);
}

// These commands scroll the display without changing the RAM
void ClcdPlus::scrollDisplayLeft(void) {
  command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVELEFT);
}

void ClcdPlus::scrollDisplayRight(void) {
  command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVERIGHT);
}

// This is for text that flows Left to Right
void ClcdPlus::leftToRight(void) {
  _displaymode |= LCD_ENTRYLEFT;
  command(LCD_ENTRYMODESET | _displaymode);
}

// This is for text that flows Right to Left
void ClcdPlus::rightToLeft(void) {
  _displaymode &= ~LCD_ENTRYLEFT;
  command(LCD_ENTRYMODESET | _displaymode);
}

// This will 'right justify' text from the cursor
void ClcdPlus::autoscroll(void) {
  _displaymode |= LCD_ENTRYSHIFTINCREMENT;
  command(LCD_ENTRYMODESET | _displaymode);
}

// This will 'left justify' text from the cursor
void ClcdPlus::noAutoscroll(void) {
  _displaymode &= ~LCD_ENTRYSHIFTINCREMENT;
  command(LCD_ENTRYMODESET | _displaymode);
}

// Allows us to fill the first 8 CGRAM locations
// with custom characters
void ClcdPlus::createChar(uint8_t location, uint8_t charmap[]) {
  location &= 0x7; // we only have 8 locations 0-7
  command(LCD_SETCGRAMADDR | (location << 3));
  for (int i=0; i<8; i++) {
    write(charmap[i]);
  }
  home() ;
}

/*********** mid level commands, for sending data/cmds */

inline void ClcdPlus::command(uint8_t value) {
  send(value, LOW);
}

inline size_t ClcdPlus::write(uint8_t value) {
  send(value, HIGH);
  return 1 ; // assume sucess
}


/************ low level data pushing commands **********/

// write either command or data, with automatic 8-bit selection
void
ClcdPlus::send(uint8_t value, uint8_t mode) {
  _state = (_state & 0xfe) | (mode ? 1: 0) ;

  gpio.write(GPIO_GPIOB, _state) ;
  gpio.write(GPIO_GPIOA, value) ;

  pulseEnable();
}

void
ClcdPlus::pulseEnable(void) {
  digitalWrite(8, LOW);
  delayMicroseconds(1);
  digitalWrite(8, HIGH);
  delayMicroseconds(1);    // enable pulse must be >450ns
  digitalWrite(8, LOW);
  delayMicroseconds(100);   // commands need > 37us to settle
}

/*
	ǉ@\
*/
void
ClcdPlus::backLight(uint8_t value) {
  if (value) {
    _state |= 0x04 ;
  } else {
    _state &= ~0x04 ;
  }
  
  gpio.write(GPIO_GPIOB, _state) ;
}

void
ClcdPlus::led(uint8_t value) {
  if (value) {
    _state |= 0x08 ;
  } else {
    _state &= ~0x08 ;
  }
  
  gpio.write(GPIO_GPIOB, _state) ;
}

uint8_t
ClcdPlus::sw1(void) {
  uint8_t v = gpio.read(GPIO_GPIOB) ;

  return (v & 0x10) ? LOW : HIGH;
}

uint8_t
ClcdPlus::sw2(void) {
  uint8_t v = gpio.read(GPIO_GPIOB) ;

  return (v & 0x20) ? LOW : HIGH;
}

uint8_t
ClcdPlus::sw3(void) {
  uint8_t v = gpio.read(GPIO_GPIOB) ;

  return (v & 0x40) ? LOW : HIGH;
}

uint8_t
ClcdPlus::sw4(void) {
  uint8_t v = gpio.read(GPIO_GPIOB) ;

  return (v & 0x80) ? LOW : HIGH;
}

uint8_t
ClcdPlus::swAll(void) {
  uint8_t v = gpio.read(GPIO_GPIOB) ;

  return ~v >> 4;
}

