/*************************************************
		ClcdPlus.h
		Ver. 1.10

	Copyright (C) 2010-2015,
		MicroFan, Div. PeopleMedia, Inc.
	All rights reserved.
*************************************************/


#ifndef ClcdPlus_h
#define ClcdPlus_h

#include <inttypes.h>
#include "Print.h"

#define	SOUNDER	9

/*************************************************
* SpiGpio
*************************************************/

#define  SPI_GPIO_CS  10
#define  SPI_GPIO_ADDR  0

#define  GPIO_IODIRA  0x00
#define  GPIO_IODIRB  0x01
#define  GPIO_IOCON  0x0A
#define  GPIO_GPPUA  0x0C
#define  GPIO_GPPUB  0x0D
#define  GPIO_GPIOA  0x12
#define  GPIO_GPIOB  0x13
#define  GPIO_OLATA  0x14
#define  GPIO_OLATB  0x15

#define GPIO_READ   0x41
#define GPIO_WRITE  0x40

class SpiGpio {
public:
    SpiGpio(uint8_t, uint8_t) ;
    void write(uint8_t, uint8_t) ;
    uint8_t read(uint8_t) ;
private:
    uint8_t _cs ;
    uint8_t devAddr ;
    uint8_t access_addr8(uint8_t, uint8_t, uint8_t) ;
} ;


/*************************************************
* ClcdPlus
	݊̎̂LiquidCrystalCp
*************************************************/

// commands
#define LCD_CLEARDISPLAY 0x01
#define LCD_RETURNHOME 0x02
#define LCD_ENTRYMODESET 0x04
#define LCD_DISPLAYCONTROL 0x08
#define LCD_CURSORSHIFT 0x10
#define LCD_FUNCTIONSET 0x20
#define LCD_SETCGRAMADDR 0x40
#define LCD_SETDDRAMADDR 0x80

// flags for display entry mode
#define LCD_ENTRYRIGHT 0x00
#define LCD_ENTRYLEFT 0x02
#define LCD_ENTRYSHIFTINCREMENT 0x01
#define LCD_ENTRYSHIFTDECREMENT 0x00

// flags for display on/off control
#define LCD_DISPLAYON 0x04
#define LCD_DISPLAYOFF 0x00
#define LCD_CURSORON 0x02
#define LCD_CURSOROFF 0x00
#define LCD_BLINKON 0x01
#define LCD_BLINKOFF 0x00

// flags for display/cursor shift
#define LCD_DISPLAYMOVE 0x08
#define LCD_CURSORMOVE 0x00
#define LCD_MOVERIGHT 0x04
#define LCD_MOVELEFT 0x00

// flags for function set
#define LCD_8BITMODE 0x10
#define LCD_4BITMODE 0x00
#define LCD_2LINE 0x08
#define LCD_1LINE 0x00
#define LCD_5x10DOTS 0x04
#define LCD_5x8DOTS 0x00

class ClcdPlus : public Print {
public:
  ClcdPlus();

  void begin(uint8_t cols, uint8_t rows, uint8_t charsize = LCD_5x8DOTS);

  void clear();
  void home();

  void noDisplay();
  void display();
  void noBlink();
  void blink();
  void noCursor();
  void cursor();
  void scrollDisplayLeft();
  void scrollDisplayRight();
  void leftToRight();
  void rightToLeft();
  void autoscroll();
  void noAutoscroll();

  void backLight(uint8_t);
  void led(uint8_t);
  uint8_t sw1(void);
  uint8_t sw2(void);
  uint8_t sw3(void);
  uint8_t sw4(void);
  uint8_t swAll(void);

  void createChar(uint8_t, uint8_t[]);
  void setCursor(uint8_t, uint8_t);

  virtual size_t write(uint8_t);
  void command(uint8_t);

  using Print::write;

private:
  SpiGpio gpio ;

  void send(uint8_t, uint8_t);
  void pulseEnable();

  uint8_t _state;

  uint8_t _displayfunction;
  uint8_t _displaycontrol;
  uint8_t _displaymode;

  uint8_t _initialized;

  uint8_t _numlines,_currline;
};

#endif
