/*
ClcdPlus Library - TextDirection

Demonstrates the use a 16x2 LCD display.  The LiquidCrystal
library works with all LCD displays that are compatible with the
Hitachi HD44780 driver. There are many of them out there, and you
can usually tell them by the 16-pin interface.

This sketch demonstrates how to use leftToRight() and rightToLeft()
to move the cursor.

Library originally added 18 Apr 2008
by David A. Mellis
library modified 5 Jul 2009
by Limor Fried (http://www.ladyada.net)
example added 9 Jul 2009
by Tom Igoe
modified 22 Nov 2010
by Tom Igoe
modified 1 Dec 2015
by Kenichi KAKIZAKI

This example code is in the public domain.

http://www.arduino.cc/en/Tutorial/LiquidCrystalTextDirection

*/

// include the library code:
#include <ClcdPlus.h>

#define JAPANESE

// initialize the library
ClcdPlus lcd;

#ifdef JAPANESE
#define	CHa	'\261'
#define	CHm	'\300'
#define	CHs	'\317'
#define	CHz	'\335'
#else
#define	CHa	'a'
#define	CHm	'm'
#define	CHs	's'
#define	CHz	'z'
#endif

int thisChar = CHa ;

void setup() {
  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  // turn on the cursor:
  lcd.cursor();
}

void loop() {
  // reverse directions at CHm:
  if (thisChar == CHm) {
    // go right for the next letter
    lcd.rightToLeft();
  }
  // reverse again at CHs:
  if (thisChar == CHs) {
    // go left for the next letter
    lcd.leftToRight();
  }
  // reset at CHz:
  if (thisChar > CHz) {
    // go to (0,0):
    lcd.home();
    // start again at 0
    thisChar = CHa;
  }

  // print the character
  lcd.write(thisChar);
  // wait half second:
  delay(500);
  // increment the letter:
  thisChar++;
}
